@@ -151,8 +151,8 @@ static void callback_read_receipt(Tox *tox, int friendnumber, uint32_t receipt,
 static void callback_connection_status(Tox *tox, int friendnumber,
     uint8_t status, void* self)
 {
-  PyObject_CallMethod((PyObject*)self, "on_connection_status", "ii",
-      friendnumber, status);
+  PyObject_CallMethod((PyObject*)self, "on_connection_status", "iO",
+      friendnumber, PyBool_FromLong(status));
 }
 
 static void callback_group_invite(Tox *tox, int friendnumber,
@@ -280,13 +280,13 @@ ToxCore_getaddress(ToxCore* self, PyObject* args)
 
   tox_get_address(self->tox, address);
   bytes_to_hex_string(address, TOX_FRIEND_ADDRESS_SIZE, address_hex);
-  
+
   PyObject* res = PyUnicodeString_FromString((const char*)address_hex);
   return res;
 }
 
 static PyObject*
-ToxCore_addfriend(ToxCore* self, PyObject* args)
+ToxCore_add_friend(ToxCore* self, PyObject* args)
 {
   uint8_t* address = NULL;
   int addr_length = 0;
@@ -343,7 +343,7 @@ ToxCore_addfriend(ToxCore* self, PyObject* args)
 }
 
 static PyObject*
-ToxCore_addfriend_norequest(ToxCore* self, PyObject* args)
+ToxCore_add_friend_norequest(ToxCore* self, PyObject* args)
 {
   uint8_t* address = NULL;
   int addr_length = 0;
@@ -364,17 +364,17 @@ ToxCore_addfriend_norequest(ToxCore* self, PyObject* args)
 }
 
 static PyObject*
-ToxCore_getfriend_id(ToxCore* self, PyObject* args)
+ToxCore_get_friend_id(ToxCore* self, PyObject* args)
 {
-  uint8_t* address = NULL;
+  uint8_t* id = NULL;
   int addr_length = 0;
 
-  if (!PyArg_ParseTuple(args, "s#", &address, &addr_length)) {
+  if (!PyArg_ParseTuple(args, "s#", &id, &addr_length)) {
     return NULL;
   }
 
-  uint8_t pk[TOX_FRIEND_ADDRESS_SIZE];
-  hex_string_to_bytes(address, TOX_FRIEND_ADDRESS_SIZE, pk);
+  uint8_t pk[TOX_CLIENT_ID_SIZE];
+  hex_string_to_bytes(id, TOX_CLIENT_ID_SIZE, pk);
 
   int ret = tox_get_friend_id(self->tox, pk);
   if (ret == -1) {
@@ -386,12 +386,12 @@ ToxCore_getfriend_id(ToxCore* self, PyObject* args)
 }
 
 static PyObject*
-ToxCore_getclient_id(ToxCore* self, PyObject* args)
+ToxCore_get_client_id(ToxCore* self, PyObject* args)
 {
-  uint8_t pk[TOX_FRIEND_ADDRESS_SIZE + 1];
+  uint8_t pk[TOX_CLIENT_ID_SIZE + 1];
   pk[TOX_FRIEND_ADDRESS_SIZE] = 0;
 
-  uint8_t hex[TOX_FRIEND_ADDRESS_SIZE * 2 + 1];
+  uint8_t hex[TOX_CLIENT_ID_SIZE * 2 + 1];
 
   int friendid = 0;
 
@@ -400,7 +400,7 @@ ToxCore_getclient_id(ToxCore* self, PyObject* args)
   }
 
   tox_get_client_id(self->tox, friendid, pk);
-  bytes_to_hex_string(pk, TOX_FRIEND_ADDRESS_SIZE, hex);
+  bytes_to_hex_string(pk, TOX_CLIENT_ID_SIZE, hex);
 
   return PyUnicodeString_FromString((const char*)hex);
 }
@@ -423,7 +423,7 @@ ToxCore_delfriend(ToxCore* self, PyObject* args)
 }
 
 static PyObject*
-ToxCore_get_friend_connectionstatus(ToxCore* self, PyObject* args)
+ToxCore_get_friend_connection_status(ToxCore* self, PyObject* args)
 {
   int friendid = 0;
 
@@ -497,7 +497,7 @@ ToxCore_sendmessage_withid(ToxCore* self, PyObject* args)
 }
 
 static PyObject*
-ToxCore_sendaction(ToxCore* self, PyObject* args)
+ToxCore_send_action(ToxCore* self, PyObject* args)
 {
   int friendid = 0;
   int length = 0;
@@ -517,7 +517,7 @@ ToxCore_sendaction(ToxCore* self, PyObject* args)
 }
 
 static PyObject*
-ToxCore_sendaction_withid(ToxCore* self, PyObject* args)
+ToxCore_send_action_withid(ToxCore* self, PyObject* args)
 {
   int friendid = 0;
   int length = 0;
@@ -539,7 +539,7 @@ ToxCore_sendaction_withid(ToxCore* self, PyObject* args)
 }
 
 static PyObject*
-ToxCore_setname(ToxCore* self, PyObject* args)
+ToxCore_set_name(ToxCore* self, PyObject* args)
 {
   uint8_t* name = 0;
   int length = 0;
@@ -557,7 +557,7 @@ ToxCore_setname(ToxCore* self, PyObject* args)
 }
 
 static PyObject*
-ToxCore_getselfname(ToxCore* self, PyObject* args)
+ToxCore_get_self_name(ToxCore* self, PyObject* args)
 {
   uint8_t buf[TOX_MAX_NAME_LENGTH];
   memset(buf, 0, TOX_MAX_NAME_LENGTH);
@@ -571,7 +571,7 @@ ToxCore_getselfname(ToxCore* self, PyObject* args)
 }
 
 static PyObject*
-ToxCore_getname(ToxCore* self, PyObject* args)
+ToxCore_get_name(ToxCore* self, PyObject* args)
 {
   int friendid = 0;
   uint8_t buf[TOX_MAX_NAME_LENGTH];
@@ -590,7 +590,7 @@ ToxCore_getname(ToxCore* self, PyObject* args)
 }
 
 static PyObject*
-ToxCore_set_statusmessage(ToxCore* self, PyObject* args)
+ToxCore_set_status_message(ToxCore* self, PyObject* args)
 {
   uint8_t* message;
   int length;
@@ -625,7 +625,7 @@ ToxCore_set_userstatus(ToxCore* self, PyObject* args)
 
 
 static PyObject*
-ToxCore_get_statusmessage_size(ToxCore* self, PyObject* args)
+ToxCore_get_status_message_size(ToxCore* self, PyObject* args)
 {
   int friendid = 0;
   if (!PyArg_ParseTuple(args, "i", &friendid)) {
@@ -633,11 +633,11 @@ ToxCore_get_statusmessage_size(ToxCore* self, PyObject* args)
   }
 
   int ret = tox_get_status_message_size(self->tox, friendid);
-  return PyLong_FromLong(ret);
+  return PyLong_FromLong(ret - 1);
 }
 
 static PyObject*
-ToxCore_copy_statusmessage(ToxCore* self, PyObject* args)
+ToxCore_get_status_message(ToxCore* self, PyObject* args)
 {
   uint8_t buf[TOX_MAX_STATUSMESSAGE_LENGTH];
   int friendid = 0;
@@ -652,7 +652,7 @@ ToxCore_copy_statusmessage(ToxCore* self, PyObject* args)
       TOX_MAX_STATUSMESSAGE_LENGTH);
 
   if (ret == -1) {
-    PyErr_SetString(ToxCoreError, "failed to copy status_message");
+    PyErr_SetString(ToxCoreError, "failed to get status_message");
     return NULL;
   }
 
@@ -662,7 +662,7 @@ ToxCore_copy_statusmessage(ToxCore* self, PyObject* args)
 }
 
 static PyObject*
-ToxCore_copy_self_statusmessage(ToxCore* self, PyObject* args)
+ToxCore_get_self_status_message(ToxCore* self, PyObject* args)
 {
   uint8_t buf[TOX_MAX_STATUSMESSAGE_LENGTH];
   memset(buf, 0, TOX_MAX_STATUSMESSAGE_LENGTH);
@@ -671,7 +671,7 @@ ToxCore_copy_self_statusmessage(ToxCore* self, PyObject* args)
       TOX_MAX_STATUSMESSAGE_LENGTH);
 
   if (ret == -1) {
-    PyErr_SetString(ToxCoreError, "failed to copy self status_message");
+    PyErr_SetString(ToxCoreError, "failed to get self status_message");
     return NULL;
   }
 
@@ -681,7 +681,7 @@ ToxCore_copy_self_statusmessage(ToxCore* self, PyObject* args)
 }
 
 static PyObject*
-ToxCore_get_userstatus(ToxCore* self, PyObject* args)
+ToxCore_get_user_status(ToxCore* self, PyObject* args)
 {
   int friendid = 0;
   if (!PyArg_ParseTuple(args, "i", &friendid)) {
@@ -694,7 +694,7 @@ ToxCore_get_userstatus(ToxCore* self, PyObject* args)
 }
 
 static PyObject*
-ToxCore_get_selfuserstatus(ToxCore* self, PyObject* args)
+ToxCore_get_self_user_status(ToxCore* self, PyObject* args)
 {
   int status = tox_get_self_user_status(self->tox);
   return PyLong_FromLong(status);
@@ -723,7 +723,7 @@ ToxCore_count_friendlist(ToxCore* self, PyObject* args)
 }
 
 static PyObject*
-ToxCore_copy_friendlist(ToxCore* self, PyObject* args)
+ToxCore_get_friendlist(ToxCore* self, PyObject* args)
 {
   PyObject* plist = NULL;
   uint32_t count = tox_count_friendlist(self->tox);
@@ -859,7 +859,7 @@ ToxCore_group_number_peers(ToxCore* self, PyObject* args)
 }
 
 static PyObject*
-ToxCore_group_copy_names(ToxCore* self, PyObject* args)
+ToxCore_group_get_names(ToxCore* self, PyObject* args)
 {
   int groupid = 0;
   if (!PyArg_ParseTuple(args, "i", &groupid)) {
@@ -868,11 +868,11 @@ ToxCore_group_copy_names(ToxCore* self, PyObject* args)
 
   int n = tox_group_number_peers(self->tox, groupid);
   uint8_t (*names)[TOX_MAX_NAME_LENGTH] = (uint8_t(*)[TOX_MAX_NAME_LENGTH])
-    malloc(sizeof(uint8_t*) * n);
+    malloc(sizeof(uint8_t*) * n * TOX_MAX_NAME_LENGTH);
 
   int n2 = tox_group_get_names(self->tox, groupid, names, n);
   if (n2 == -1) {
-    PyErr_SetString(PyExc_TypeError, "failed to copy group member names");
+    PyErr_SetString(PyExc_TypeError, "failed to get group member names");
     return NULL;
   }
 
@@ -886,6 +886,8 @@ ToxCore_group_copy_names(ToxCore* self, PyObject* args)
     PyList_Append(list, PyUnicodeString_FromString((const char*)names[i]));
   }
 
+  free(names);
+
   return list;
 }
 
@@ -897,7 +899,7 @@ ToxCore_count_chatlist(ToxCore* self, PyObject* args)
 }
 
 static PyObject*
-ToxCore_copy_chatlist(ToxCore* self, PyObject* args)
+ToxCore_get_chatlist(ToxCore* self, PyObject* args)
 {
   PyObject* plist = NULL;
   uint32_t count = tox_count_chatlist(self->tox);
@@ -953,7 +955,7 @@ ToxCore_file_sendcontrol(ToxCore* self, PyObject* args)
   int data_length = 0;
 
 
-  if (!PyArg_ParseTuple(args, "ibibs#", &friendnumber, &send_receive,
+  if (!PyArg_ParseTuple(args, "ibib|s#", &friendnumber, &send_receive,
         &filenumber, &message_id, &data, &data_length)) {
     return NULL;
   }
@@ -995,7 +997,7 @@ ToxCore_file_senddata(ToxCore* self, PyObject* args)
 }
 
 static PyObject*
-ToxCore_filedata_size(ToxCore* self, PyObject* args)
+ToxCore_file_data_size(ToxCore* self, PyObject* args)
 {
   int friendnumber = 0;
   if (!PyArg_ParseTuple(args, "i", &friendnumber)) {
@@ -1208,132 +1210,400 @@ ToxCore_load_from_file(ToxCore* self, PyObject* args)
 }
 
 PyMethodDef Tox_methods[] = {
-  {"on_friend_request", (PyCFunction)ToxCore_callback_stub, METH_VARARGS, ""},
-  {"on_friend_message", (PyCFunction)ToxCore_callback_stub, METH_VARARGS, ""},
-  {"on_action", (PyCFunction)ToxCore_callback_stub, METH_VARARGS, ""},
-  {"on_name_change", (PyCFunction)ToxCore_callback_stub, METH_VARARGS, ""},
-  {"on_status_message", (PyCFunction)ToxCore_callback_stub, METH_VARARGS, ""},
-  {"on_user_status", (PyCFunction)ToxCore_callback_stub, METH_VARARGS, ""},
-  {"on_read_receipt", (PyCFunction)ToxCore_callback_stub, METH_VARARGS, ""},
-  {"on_connection_status", (PyCFunction)ToxCore_callback_stub, METH_VARARGS, ""},
-  {"on_group_invite", (PyCFunction)ToxCore_callback_stub, METH_VARARGS, ""},
-  {"on_group_message", (PyCFunction)ToxCore_callback_stub, METH_VARARGS, ""},
-  {"on_group_namelist_change", (PyCFunction)ToxCore_callback_stub, METH_VARARGS, ""},
-  {"on_file_send_request", (PyCFunction)ToxCore_callback_stub, METH_VARARGS, ""},
-  {"on_file_control", (PyCFunction)ToxCore_callback_stub, METH_VARARGS, ""},
-  {"on_file_data", (PyCFunction)ToxCore_callback_stub, METH_VARARGS, ""},
-  {"get_address", (PyCFunction)ToxCore_getaddress, METH_NOARGS,
-    "return FRIEND_ADDRESS_SIZE byte address to give to others" },
-  {"add_friend", (PyCFunction)ToxCore_addfriend, METH_VARARGS,
-    "add a friend" },
-  {"add_friend_norequest", (PyCFunction)ToxCore_addfriend_norequest,
-    METH_VARARGS,
-    "add a friend without sending request" },
-  {"get_friend_id", (PyCFunction)ToxCore_getfriend_id, METH_VARARGS,
-    "return the friend id associated to that client id" },
-  {"get_client_id", (PyCFunction)ToxCore_getclient_id, METH_VARARGS,
-    "Copies the public key associated to that friend id into client_id buffer"
+  {
+    "on_friend_request", (PyCFunction)ToxCore_callback_stub, METH_VARARGS,
+    "on_friend_request(address, message)\n"
+    "Callback for receiving friend requests, default implementation does "
+    "nothing."
   },
-  {"del_friend", (PyCFunction)ToxCore_delfriend, METH_VARARGS,
-    "Remove a friend" },
-  {"get_friend_connection_status",
-    (PyCFunction)ToxCore_get_friend_connectionstatus, METH_VARARGS,
-    "Checks friend's connecting status" },
-  {"friend_exists", (PyCFunction)ToxCore_friend_exists, METH_VARARGS,
-    "Checks if there exists a friend with given friendnumber" },
-  {"send_message", (PyCFunction)ToxCore_sendmessage, METH_VARARGS,
-    "Send a text chat message to an online friend" },
-  {"send_message_withid", (PyCFunction)ToxCore_sendmessage_withid, METH_VARARGS,
-    "Send a text chat message to an online friend with id" },
-  {"send_action", (PyCFunction)ToxCore_sendaction, METH_VARARGS,
-    "Send an action to an online friend" },
-  {"send_action_withid", (PyCFunction)ToxCore_sendaction_withid, METH_VARARGS,
-    "Send an action to an online friend with id" },
-  {"set_name", (PyCFunction)ToxCore_setname, METH_VARARGS,
-    "Set our nickname" },
-  {"get_self_name", (PyCFunction)ToxCore_getselfname, METH_NOARGS,
-    "Get your nickname" },
-  {"get_name", (PyCFunction)ToxCore_getname, METH_VARARGS,
-    "Get name of friendnumber and put it in name" },
-  {"set_status_message", (PyCFunction)ToxCore_set_statusmessage, METH_VARARGS,
-    "Set our user status message" },
-  {"set_userstatus", (PyCFunction)ToxCore_set_userstatus, METH_VARARGS,
-    "Set our user status" },
-  {"get_status_message_size", (PyCFunction)ToxCore_get_statusmessage_size,
+  {
+    "on_friend_message", (PyCFunction)ToxCore_callback_stub, METH_VARARGS,
+    "on_friend_message(friend_number, message)\n"
+    "Callback for receiving friend messages, default implementation does "
+    "nothing."
+  },
+  {
+    "on_action", (PyCFunction)ToxCore_callback_stub, METH_VARARGS,
+    "on_action(friend_number, action)\n"
+    "Callback for receiving friend actions, default implementation does "
+    "nothing."
+  },
+  {
+    "on_name_change", (PyCFunction)ToxCore_callback_stub, METH_VARARGS,
+    "on_name_change(friend_number, new_name)\n"
+    "Callback for receiving friend name changes, default implementation does "
+    "nothing."
+  },
+  {
+    "on_status_message", (PyCFunction)ToxCore_callback_stub, METH_VARARGS,
+    "on_status_message(friend_number, new_status)\n"
+    "Callback for receiving friend status message changes, default "
+    "implementation does nothing."
+  },
+  {
+    "on_user_status", (PyCFunction)ToxCore_callback_stub, METH_VARARGS,
+    "on_user_status(friend_number, kind)\n"
+    "Callback for receiving friend status changes, default implementation "
+    "does nothing.\n\n"
+    ".. seealso ::\n"
+    "    :meth:`.set_userstatus`"
+  },
+  {
+    "on_read_receipt", (PyCFunction)ToxCore_callback_stub, METH_VARARGS,
+    "on_read_receipt(friend_number, receipt)\n"
+    "Callback for receiving read receipt, default implementation does nothing."
+  },
+  {
+    "on_connection_status", (PyCFunction)ToxCore_callback_stub, METH_VARARGS,
+    "on_connection_status(friend_number, status)\n"
+    "Callback for receiving read receipt, default implementation does "
+    "nothing.\n\n"
+    "*status* is a boolean value which indicates the status of the friend "
+    "indicated by *friend_number*. True means online and False means offline "
+    "after previously online."
+  },
+  {
+    "on_group_invite", (PyCFunction)ToxCore_callback_stub, METH_VARARGS,
+    "on_group_invite(friend_number, group_public_key)\n"
+    "Callback for receiving group invitations, default implementation does "
+    "nothing."
+  },
+  {
+    "on_group_message", (PyCFunction)ToxCore_callback_stub, METH_VARARGS,
+    "on_group_message(group_number, friend_group_number, message)\n"
+    "Callback for receiving group messages, default implementation does "
+    "nothing."
+  },
+  {
+    "on_group_namelist_change", (PyCFunction)ToxCore_callback_stub,
     METH_VARARGS,
-    "return the length of friendnumber's status message, including null" },
-  {"get_status_message", (PyCFunction)ToxCore_copy_statusmessage,
+    "on_group_namelist_change(group_number, peer_number, change)\n"
+    "Callback for receiving group messages, default implementation does "
+    "nothing.\n\n"
+    "There are there possible *change* values:\n\n"
+    "+---------------------------+----------------------+\n"
+    "| change                    | description          |\n"
+    "+===========================+======================+\n"
+    "| Tox.CHAT_CHANGE_PEER_ADD  | a peer is added      |\n"
+    "+---------------------------+----------------------+\n"
+    "| Tox.CHAT_CHANGE_PEER_DEL  | a peer is deleted    |\n"
+    "+---------------------------+----------------------+\n"
+    "| Tox.CHAT_CHANGE_PEER_NAME | name of peer changed |\n"
+    "+---------------------------+----------------------+\n"
+  },
+  {
+    "on_file_send_request", (PyCFunction)ToxCore_callback_stub, METH_VARARGS,
+    "on_file_send_request(friend_number, file_number, file_size, filename)\n"
+    "Callback for receiving file send requests, default implementation does "
+    "nothing."
+  },
+  {
+    "on_file_control", (PyCFunction)ToxCore_callback_stub, METH_VARARGS,
+    "on_file_control(friend_number, receive_send, file_number, control_type, "
+    "data)\n"
+    "Callback for receiving file send control, default implementation does "
+    "nothing. See :meth:`.file_send_control` for the meaning of *receive_send* "
+    "and *control_type*.\n\n"
+    ".. seealso ::\n"
+    "    :meth:`.file_send_control`"
+  },
+  {
+    "on_file_data", (PyCFunction)ToxCore_callback_stub, METH_VARARGS,
+    "on_file_data(friend_number, file_number, data)\n"
+    "Callback for receiving file data, default implementation does nothing."
+  },
+  {
+    "get_address", (PyCFunction)ToxCore_getaddress, METH_NOARGS,
+    "get_address()\n"
+    "Return address to give to others."
+  },
+  {
+    "add_friend", (PyCFunction)ToxCore_add_friend, METH_VARARGS,
+    "add_friend(address, message)\n"
+    "Add a friend."
+  },
+  {
+    "add_friend_norequest", (PyCFunction)ToxCore_add_friend_norequest,
     METH_VARARGS,
-    "get status message of a friend" },
-  {"get_selfstatusmessage", (PyCFunction)ToxCore_copy_self_statusmessage,
+    "add_friend_norequest(address)\n"
+    "Add a friend without sending request."
+  },
+  {
+    "get_friend_id", (PyCFunction)ToxCore_get_friend_id, METH_VARARGS,
+    "get_friend_id(friend_id)\n"
+    "Return the friend id associated to that client id."
+  },
+  {
+    "get_client_id", (PyCFunction)ToxCore_get_client_id, METH_VARARGS,
+    "get_client_id(friend_number)\n"
+    "Return the public key associated to that friend number."
+  },
+  {
+    "del_friend", (PyCFunction)ToxCore_delfriend, METH_VARARGS,
+    "del_friend(friend_number)\n"
+    "Remove a friend."
+  },
+  {
+    "get_friend_connection_status",
+    (PyCFunction)ToxCore_get_friend_connection_status, METH_VARARGS,
+    "get_friend_connection_status(friend_number)\n"
+    "Return True if friend is connected(Online) else False."
+  },
+  {
+    "friend_exists", (PyCFunction)ToxCore_friend_exists, METH_VARARGS,
+    "friend_exists(friend_number)\n"
+    "Checks if there exists a friend with given friendnumber."
+  },
+  {
+    "send_message", (PyCFunction)ToxCore_sendmessage, METH_VARARGS,
+    "send_message(friend_number, message)\n"
+    "Send a text chat message to an online friend."
+  },
+  {
+    "send_message_withid", (PyCFunction)ToxCore_sendmessage_withid,
+    METH_VARARGS,
+    "send_message_withid(friend_number, id, message)\n"
+    "Send a text chat message to an online friend with id."
+  },
+  {
+    "send_action", (PyCFunction)ToxCore_send_action, METH_VARARGS,
+    "send_action(friend_number, action)\n"
+    "Send an action to an online friend."
+  },
+  {
+    "send_action_withid", (PyCFunction)ToxCore_send_action_withid, METH_VARARGS,
+    "send_action_withid(friend_number, id, action)\n"
+    "Send an action to an online friend with id."
+  },
+  {
+    "set_name", (PyCFunction)ToxCore_set_name, METH_VARARGS,
+    "set_name(name)\n"
+    "Set our nickname."
+  },
+  {
+    "get_self_name", (PyCFunction)ToxCore_get_self_name, METH_NOARGS,
+    "get_self_name()\n"
+    "Get your nickname."
+  },
+  {
+    "get_name", (PyCFunction)ToxCore_get_name, METH_VARARGS,
+    "get_name(friend_number)\n"
+    "Get name of *friend_number*."
+  },
+  {
+    "set_status_message", (PyCFunction)ToxCore_set_status_message, METH_VARARGS,
+    "set_status_message(message)\n"
+    "Set our user status message."
+  },
+  {
+    "set_userstatus", (PyCFunction)ToxCore_set_userstatus, METH_VARARGS,
+    "set_userstatus(status)\n"
+    "Set our user status, status can have following values:\n\n"
+    "+------------------------+--------------------+\n"
+    "| kind                   | description        |\n"
+    "+========================+====================+\n"
+    "| Tox.USERSTATUS_NONE    | the user is online |\n"
+    "+------------------------+--------------------+\n"
+    "| Tox.USERSTATUS_AWAY    | the user is away   |\n"
+    "+------------------------+--------------------+\n"
+    "| Tox.USERSTATUS_BUSY    | the user is busy   |\n"
+    "+------------------------+--------------------+\n"
+    "| Tox.USERSTATUS_INVALID | invalid status     |\n"
+    "+------------------------+--------------------+\n"
+  },
+  {
+    "get_status_message_size", (PyCFunction)ToxCore_get_status_message_size,
+    METH_VARARGS,
+    "get_status_message_size(friend_number)\n"
+    "Return the length of *friend_number*'s status message."
+  },
+  {
+    "get_status_message", (PyCFunction)ToxCore_get_status_message,
+    METH_VARARGS,
+    "get_status_message(friend_number)\n"
+    "Get status message of a friend."
+  },
+  {
+    "get_self_status_message", (PyCFunction)ToxCore_get_self_status_message,
     METH_NOARGS,
-    "get status message of yourself" },
-  {"get_user_status", (PyCFunction)ToxCore_get_userstatus,
+    "get_self_status_message()\n"
+    "Get status message of yourself."
+  },
+  {
+    "get_user_status", (PyCFunction)ToxCore_get_user_status,
     METH_VARARGS,
-    "get friend status" },
-  {"get_self_user_status", (PyCFunction)ToxCore_get_selfuserstatus,
+    "get_user_status(friend_number)\n"
+    "Get friend status.\n\n"
+    ".. seealso ::\n"
+    "    :meth:`.set_userstatus`"
+  },
+  {
+    "get_self_user_status", (PyCFunction)ToxCore_get_self_user_status,
     METH_VARARGS,
-    "get self user_status" },
-  {"set_sends_receipts", (PyCFunction)ToxCore_set_send_receipts,
+    "get_self_user_status()\n"
+    "Get user status of youself.\n\n"
+    ".. seealso ::\n"
+    "    :meth:`.set_userstatus`"
+  },
+  {
+    "set_sends_receipts", (PyCFunction)ToxCore_set_send_receipts,
     METH_VARARGS,
-    "Sets whether we send read receipts for friendnumber." },
-  {"count_friendlist", (PyCFunction)ToxCore_count_friendlist,
+    "set_sends_receipts(friend_number, yesno)\n"
+    "Sets whether we send read receipts for friendnumber. *yesno* should be "
+    "a boolean value."
+  },
+  {
+    "count_friendlist", (PyCFunction)ToxCore_count_friendlist,
     METH_NOARGS,
-    "Return the number of friends" },
-  {"get_friendlist", (PyCFunction)ToxCore_copy_friendlist,
+    "count_friendlist()\n"
+    "Return the number of friends."
+  },
+  {
+    "get_friendlist", (PyCFunction)ToxCore_get_friendlist,
     METH_NOARGS,
-    "Copy a list of valid friend IDs into the array out_list" },
-  {"add_groupchat", (PyCFunction)ToxCore_add_groupchat, METH_VARARGS,
-    "Creates a new groupchat and puts it in the chats array"},
-  {"del_groupchat", (PyCFunction)ToxCore_del_groupchat, METH_VARARGS,
-    "Delete a groupchat from the chats array"},
-  {"group_peername", (PyCFunction)ToxCore_group_peername, METH_VARARGS,
-    "get the group peer's name"},
-  {"invite_friend", (PyCFunction)ToxCore_invite_friend, METH_VARARGS,
-    "invite friendnumber to groupnumber "},
-  {"join_groupchat", (PyCFunction)ToxCore_join_groupchat, METH_VARARGS,
-    "Join a group (you need to have been invited first.)"},
-  {"group_message_send", (PyCFunction)ToxCore_group_message_send, METH_VARARGS,
-    "send a group message"},
-  {"group_number_peers", (PyCFunction)ToxCore_group_number_peers, METH_VARARGS,
-    "Return the number of peers in the group chat on success."},
-  {"group_get_names", (PyCFunction)ToxCore_group_copy_names, METH_VARARGS,
-    "List all the peers in the group chat."},
-  {"count_chatlist", (PyCFunction)ToxCore_count_chatlist, METH_VARARGS,
-    "Return the number of chats in the instance m."},
-  {"get_chatlist", (PyCFunction)ToxCore_copy_chatlist, METH_VARARGS,
-    "Copy a list of valid chat IDs into the array out_list."},
-  {"new_file_sender", (PyCFunction)ToxCore_new_filesender, METH_VARARGS,
-    "Send a file send request."},
-  {"file_send_control", (PyCFunction)ToxCore_file_sendcontrol, METH_VARARGS,
-    "Send a file control request."},
-  {"file_send_data", (PyCFunction)ToxCore_file_senddata, METH_VARARGS,
-    "Send file data."},
-  {"file_data_size", (PyCFunction)ToxCore_filedata_size, METH_VARARGS,
+    "get_friendlist()\n"
+    "Get a list of valid friend numbers."
+  },
+  {
+    "add_groupchat", (PyCFunction)ToxCore_add_groupchat, METH_VARARGS,
+    "add_groupchat()\n"
+    "Creates a new groupchat and puts it in the chats array."
+  },
+  {
+    "del_groupchat", (PyCFunction)ToxCore_del_groupchat, METH_VARARGS,
+    "del_groupchat(gruop_number)\n"
+    "Delete a groupchat from the chats array."
+  },
+  {
+    "group_peername", (PyCFunction)ToxCore_group_peername, METH_VARARGS,
+    "group_peername(group_number, peer_number)\n"
+    "Get the group peer's name."
+  },
+  {
+    "invite_friend", (PyCFunction)ToxCore_invite_friend, METH_VARARGS,
+    "invite_friend(friend_number, group_number)\n"
+    "Invite friendnumber to groupnumber."
+  },
+  {
+    "join_groupchat", (PyCFunction)ToxCore_join_groupchat, METH_VARARGS,
+    "join_groupchat(friend_number, publick_key)\n"
+    "Join a group (you need to have been invited first.)"
+  },
+  {
+    "group_message_send", (PyCFunction)ToxCore_group_message_send, METH_VARARGS,
+    "group_message_send(group_number, message)\n"
+    "send a group message."
+  },
+  {
+    "group_number_peers", (PyCFunction)ToxCore_group_number_peers, METH_VARARGS,
+    "group_number_peers(group_number)\n"
+    "Return the number of peers in the group chat on success."
+  },
+  {
+    "group_get_names", (PyCFunction)ToxCore_group_get_names, METH_VARARGS,
+    "group_get_names(group_number)\n"
+    "List all the peers in the group chat."
+  },
+  {
+    "count_chatlist", (PyCFunction)ToxCore_count_chatlist, METH_VARARGS,
+    "count_chatlist()\n"
+    "Return the number of chats in the current Tox instance."
+  },
+  {
+    "get_chatlist", (PyCFunction)ToxCore_get_chatlist, METH_VARARGS,
+    "get_chatlist()\n"
+    "Return a list of valid group numbers."
+  },
+  {
+    "new_file_sender", (PyCFunction)ToxCore_new_filesender, METH_VARARGS,
+    "new_file_sender(friend_number, file_size, filename)\n"
+    "Send a file send request."
+  },
+  {
+    "file_send_control", (PyCFunction)ToxCore_file_sendcontrol, METH_VARARGS,
+    "file_send_control(friend_number, send_receive, file_number, control_type"
+    "[, data=None])\n"
+    "Send file transfer control.\n\n"
+    "*send_receive* is 0 for sending and 1 for receiving.\n\n"
+    "*control_type* can be one of following value:\n\n"
+    "+-------------------------------+------------------------+\n"
+    "| control_type                  | description            |\n"
+    "+===============================+========================+\n"
+    "| Tox.FILECONTROL_ACCEPT        | accepts transfer       |\n"
+    "+-------------------------------+------------------------+\n"
+    "| Tox.FILECONTROL_PAUSE         | pause transfer         |\n"
+    "+-------------------------------+------------------------+\n"
+    "| Tox.FILECONTROL_KILL          | kill/rejct transfer    |\n"
+    "+-------------------------------+------------------------+\n"
+    "| Tox.FILECONTROL_FINISHED      | transfer finished      |\n"
+    "+-------------------------------+------------------------+\n"
+    "| Tox.FILECONTROL_RESUME_BROKEN | resume broken transfer |\n"
+    "+-------------------------------+------------------------+\n"
+  },
+  {
+    "file_send_data", (PyCFunction)ToxCore_file_senddata, METH_VARARGS,
+    "file_send_data(friend_number, file_number, data)\n"
+    "Send file data."
+  },
+  {
+    "file_data_size", (PyCFunction)ToxCore_file_data_size, METH_VARARGS,
+    "file_data_size(friend_number)\n"
     "Returns the recommended/maximum size of the filedata you send with "
-    "tox_file_send_data()"},
-  {"file_data_remaining", (PyCFunction)ToxCore_file_dataremaining, METH_VARARGS,
-    "Give the number of bytes left to be sent/received."},
-  {"bootstrap_from_address", (PyCFunction)ToxCore_bootstrap_from_address,
+    ":meth:`.file_send_data`."},
+  {
+    "file_data_remaining", (PyCFunction)ToxCore_file_dataremaining,
     METH_VARARGS,
+    "file_data_remaining(friend_number, file_number, send_receive)\n"
+    "Give the number of bytes left to be sent/received. *send_receive* is "
+    "0 for sending and 1 for receiving."
+  },
+  {
+    "bootstrap_from_address", (PyCFunction)ToxCore_bootstrap_from_address,
+    METH_VARARGS,
+    "bootstrap_from_address(address, ipv6enabled, port, public_key)\n"
     "Resolves address into an IP address. If successful, sends a 'get nodes'"
-    "request to the given node with ip, port " },
-  {"isconnected", (PyCFunction)ToxCore_isconnected, METH_NOARGS,
-    "return False if we are not connected to the DHT." },
-  {"kill", (PyCFunction)ToxCore_kill, METH_NOARGS,
-    "Run this before closing shop" },
-  {"do", (PyCFunction)ToxCore_do, METH_NOARGS,
-    "The main loop that needs to be run at least 20 times per second" },
-  {"size", (PyCFunction)ToxCore_size, METH_NOARGS,
-    "return size of messenger data (for saving)." },
-  {"save", (PyCFunction)ToxCore_save, METH_NOARGS,
-    "Save the messenger in data" },
-  {"save_to_file", (PyCFunction)ToxCore_save_to_file, METH_VARARGS,
-    "Save the messenger to a file" },
-  {"load", (PyCFunction)ToxCore_load, METH_VARARGS,
-    "Load the messenger from data of size length." },
-  {"load_from_file", (PyCFunction)ToxCore_load_from_file, METH_VARARGS,
-    "Load the messenger from file" },
+    "request to the given node with ip, port."
+  },
+  {
+    "isconnected", (PyCFunction)ToxCore_isconnected, METH_NOARGS,
+    "isconnected()\n"
+    "Return False if we are not connected to the DHT."
+  },
+  {
+    "kill", (PyCFunction)ToxCore_kill, METH_NOARGS,
+    "kill()\n"
+    "Run this before closing shop."
+  },
+  {
+    "do", (PyCFunction)ToxCore_do, METH_NOARGS,
+    "do()\n"
+    "The main loop that needs to be run at least 20 times per second."
+  },
+  {
+    "size", (PyCFunction)ToxCore_size, METH_NOARGS,
+    "size()\n"
+    "return size of messenger data (for saving)."
+  },
+  {
+    "save", (PyCFunction)ToxCore_save, METH_NOARGS,
+    "save()\n"
+    "Return messenger blob in str."
+  },
+  {
+    "save_to_file", (PyCFunction)ToxCore_save_to_file, METH_VARARGS,
+    "save_to_file(filename)\n"
+    "Save the messenger to a file."
+  },
+  {
+    "load", (PyCFunction)ToxCore_load, METH_VARARGS,
+    "load(blob)\n"
+    "Load the messenger from *blob*."
+  },
+  {
+    "load_from_file", (PyCFunction)ToxCore_load_from_file, METH_VARARGS,
+    "load_from_file(filename)\n"
+    "Load the messenger from file."
+  },
   {NULL}
 };
 
@@ -1383,4 +1653,34 @@ PyTypeObject ToxCoreType = {
   ToxCore_new,               /* tp_new */
 };
 
+void ToxCore_install_dict()
+{
+#define SET(name) \
+  PyDict_SetItemString(dict, #name, PyLong_FromLong(TOX_##name));
 
+  PyObject* dict = PyDict_New();
+  SET(FAERR_TOOLONG)
+  SET(FAERR_NOMESSAGE)
+  SET(FAERR_OWNKEY)
+  SET(FAERR_ALREADYSENT)
+  SET(FAERR_UNKNOWN)
+  SET(FAERR_BADCHECKSUM)
+  SET(FAERR_SETNEWNOSPAM)
+  SET(FAERR_NOMEM)
+  SET(USERSTATUS_NONE)
+  SET(USERSTATUS_AWAY)
+  SET(USERSTATUS_BUSY)
+  SET(USERSTATUS_INVALID)
+  SET(CHAT_CHANGE_PEER_ADD)
+  SET(CHAT_CHANGE_PEER_DEL)
+  SET(CHAT_CHANGE_PEER_NAME)
+  SET(FILECONTROL_ACCEPT)
+  SET(FILECONTROL_PAUSE)
+  SET(FILECONTROL_KILL)
+  SET(FILECONTROL_FINISHED)
+  SET(FILECONTROL_RESUME_BROKEN)
+
+#undef SET
+
+  ToxCoreType.tp_dict = dict;
+}
